#ifndef PDE2D_H
#define PDE2D_H

#include "polygon.h"
#include "point2d.h"

/** PDE2D:
 *      Provides an interface for PDE problem classes.
 */
class PDE2D{

public:
    
    PDE2D(){}
    
    /** Assembles A^(k) and b^(k) on each triangle, given by its node points,
     *  (Abstract function that has to be implemented in each derived class)
     */
    virtual void 
    assemble_on_element(Point2D& x1, Point2D& x2, Point2D& x3, double A[3][3], double b[3]) = 0;
    
    /** Returns the Dirichlet boundary condition at point x
     *  (can be overloaded, default are homogeneous boundaries)
     */
    virtual
    double
    dirichlet_bc(Point2D& x){
        return 0;
    };

    /** Returns the initial condition at point x.
     *  (necessary if we want to solve a instationary PDE)
     *  (can be overloaded, default is homogeneous initial condition)
     */
    virtual
    double
    initial_condition(Point2D& x){
        return 0;
    };
 
};


#endif // PDE2D_H