#ifndef SPARSEMATRIX_H
#define SPARSEMATRIX_H 1

#include <map>
#include <utility>



//bool operator<(const std::pair<int,int> & left, const std::pair<int, int> & right){
//    if(left.first < right.first){
//        return true;
//    } else if(left.first == right.first){
//        if(left.second < right.second){
//            return true;
//        }
//    }
//    return false;
//}


class SparseMatrix{
    
    int M,N;
    std::map<std::pair<int,int>, double> data;
    
public:
    
    SparseMatrix(const int n);

    SparseMatrix(const int m, const int n);
    
    SparseMatrix(const SparseMatrix& rhs);

    double & 
    operator()(int m,int n);

    int numCols() const;

    int numRows() const;
    
    void clear();
    
    const std::map<std::pair<int,int>, double>&
    getData() const;

    // Add alpha*rhs to this matrix
    void axpy(double alpha, const SparseMatrix& rhs);
    
    SparseMatrix&
    operator=(const SparseMatrix& rhs);
};



SparseMatrix::SparseMatrix(const int n):M(n),N(n){}


SparseMatrix::SparseMatrix(const int m, const int n):M(m),N(n){}

SparseMatrix::SparseMatrix(const SparseMatrix& rhs) : M(rhs.numRows()), N(rhs.numCols()), data(rhs.getData()){}

//this operator is used to set the non-zero values in the sparse-matrix
//CAREFUL: Do not try to access matrix positions where the value is zero, since this creates an entry with value zero!
// Then SparseMatrix is not sparse anymore!
double &
SparseMatrix::operator()(int m, int n){ 
    return data[std::pair<int,int>(m,n)];
}


int
SparseMatrix::numRows() const{
    return M;
}

int 
SparseMatrix::numCols() const{
    return N;
}

void 
SparseMatrix::clear(){
    data.clear();
}

const std::map<std::pair<int,int>, double>&
SparseMatrix::getData() const{
    return data;
}

void
SparseMatrix::axpy(double alpha, const SparseMatrix& rhs){
    
    for(std::map<std::pair<int,int>,double>::const_iterator it = rhs.getData().begin(); it != rhs.getData().end(); ++it){
        operator()((*it).first.first, (*it).first.second) += alpha * (*it).second;
    }
}

SparseMatrix&
SparseMatrix::operator=(const SparseMatrix& rhs){
    if(&rhs == this){
        return *this;
    }
    M = rhs.numRows();
    N = rhs.numCols();
    data = rhs.getData();
    return *this;
}

std::ostream& 
operator<< (std::ostream& out, SparseMatrix & matrix){
    for(std::map<std::pair<int,int>,double>::const_iterator iterator = matrix.getData().begin(); iterator != matrix.getData().end(); iterator++) {
        out << "(" << (iterator->first).first << "," << (iterator->first).second << ") => " << iterator->second << std::endl;
    }
    return out;
}

#endif 